syntax = "proto3";

import "custom_options.proto";

package test;

option (options.file_int32) = 7;

message MessageWithOptions {
    option (options.msg_string)  = "Custom message option";
    option (options.msg_integer) = -2;

    int32 int32_f = 1 [(options.fld_string) = "Custom field option"];
    string string_f = 2;
}
