syntax = "proto3";

package helloworld;

service Greeter {
    rpc SayHello (HelloRequest) returns (HelloReply);

    rpc JoinedHello (stream HelloRequest) returns (HelloReply);

    rpc SplitHello (HelloRequest) returns (stream HelloReply);

    rpc WavingHello (stream HelloRequest) returns (stream HelloReply);
}

message HelloRequest {
    string name = 1;
}

message HelloReply {
    string message = 1;
}
